#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from random import choice
from testlib import get_fixture, function, random_int, random_string
from testlib import EapiConfigUnitTest

import pyeapi.api.staticroute

IP_DESTS = ['11.111.11.0/24', '222.22.222.0/24', '33.34.35.0/24']
NEXT_HOPS = [('Ethernet1', '3.3.3.3'), ('Ethernet2', '2.2.2.2'),
             ('Null0', None), ('44.44.44.0', None)]
DISTANCES = TAGS = ROUTE_NAMES = [None, True]


class TestApiStaticroute(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiStaticroute, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.staticroute.StaticRoute(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_instance(self):
        result = pyeapi.api.staticroute.instance(None)
        self.assertIsInstance(result, pyeapi.api.staticroute.StaticRoute)

    def test_get(self):
        # Test retrieval of a specific static route entry
        # Assumes running_config.text file contains the following
        # ip route specifications, and that no additional routes
        # are specified.

        # ip route 0.0.0.0/0 192.68.1.254 1 tag 0
        # ip route 1.2.3.0/24 Ethernet1 1.1.1.1 1 tag 1 name test1
        # ip route 1.2.3.0/24 Ethernet1 1.1.1.1 10 tag 1 name test1
        # ip route 1.2.3.0/24 Ethernet1 10.1.1.1 20 tag 1 name test1

        # Get the route(s) for ip_dest 0.0.0.0/24
        ip_dest = '0.0.0.0/0'
        routes = {
            '192.68.1.254': {
                None: {
                    1: {'route_name': None,
                        'tag': 0}
                }
            }
        }
        result = self.instance.get(ip_dest)
        self.assertEqual(result, routes)

        # Get the route(s) for ip_dest 1.2.3.0/24
        ip_dest = '1.2.3.0/24'
        routes = {
            'Ethernet1': {
                '1.1.1.1': {
                    1: {
                        'route_name': 'test1',
                        'tag': 1},
                    10: {
                        'route_name': 'test1',
                        'tag': 1}
                },
                '10.1.1.1': {
                    20: {
                        'route_name': 'test1',
                        'tag': 1}
                }
            }
        }
        result = self.instance.get(ip_dest)
        self.assertEqual(result, routes)

    def test_getall(self):
        # Test retrieval of all static route entries
        # Assumes running_config.text file contains the following
        # ip route specifications, and that no additional routes
        # are specified.

        # ip route 0.0.0.0/0 192.68.1.254 1 tag 0
        # ip route 1.2.3.0/24 Ethernet1 1.1.1.1 1 tag 1 name test1
        # ip route 1.2.3.0/24 Ethernet1 1.1.1.1 10 tag 1 name test1
        # ip route 1.2.3.0/24 Ethernet1 10.1.1.1 20 tag 1 name test1

        routes = {
            '0.0.0.0/0': {
                '192.68.1.254': {
                    None: {
                        1: {'route_name': None,
                            'tag': 0}
                    }
                }
            },
            '1.2.3.0/24': {
                'Ethernet1': {
                    '1.1.1.1': {
                        1: {
                            'route_name': 'test1',
                            'tag': 1},
                        10: {
                            'route_name': 'test1',
                            'tag': 1}
                    },
                    '10.1.1.1': {
                        20: {
                            'route_name': 'test1',
                            'tag': 1}
                    }
                }
            }
        }

        self.maxDiff = None
        result = self.instance.getall()
        self.assertEqual(result, routes)

    def test_create(self):
        # Test passing in a full set of parameters to 'create'
        # Some parameters may be not set: None
        for ip_dest in IP_DESTS:
            # Get the parameters for the call
            (next_hop, next_hop_ip) = choice(NEXT_HOPS)
            distance = choice(DISTANCES)
            if distance:
                distance = random_int(0, 255)
            tag = choice(TAGS)
            if tag:
                tag = random_int(0, 255)
            route_name = choice(ROUTE_NAMES)
            if route_name:
                route_name = random_string(minchar=4, maxchar=10)

            func = function('create', ip_dest, next_hop,
                            next_hop_ip=next_hop_ip,
                            distance=distance,
                            tag=tag,
                            route_name=route_name)

            # Build the expected string for comparison
            # A value of None will default to an empty string, and
            # add the tag or name keywords where appropriate
            cmd_next_hop_ip = cmd_distance = cmd_tag = cmd_route_name = ''
            if next_hop_ip is not None:
                cmd_next_hop_ip = " %s" % next_hop_ip
            if distance is not None:
                cmd_distance = " %d" % distance
            if tag is not None:
                cmd_tag = " tag %d" % tag
            if route_name is not None:
                cmd_route_name = " name %s" % route_name
            cmds = "ip route %s %s%s%s%s%s" % \
                   (ip_dest, next_hop, cmd_next_hop_ip, cmd_distance,
                    cmd_tag, cmd_route_name)

            self.eapi_positive_config_test(func, cmds)

    def test_delete(self):
        # Test passing in a full set of parameters to 'delete'
        # Some parameters may be not set: None
        for ip_dest in IP_DESTS:
            (next_hop, next_hop_ip) = choice(NEXT_HOPS)
            distance = choice(DISTANCES)
            if distance:
                distance = random_int(0, 255)
            tag = choice(TAGS)
            if tag:
                tag = random_int(0, 255)
            route_name = choice(ROUTE_NAMES)
            if route_name:
                route_name = random_string(minchar=4, maxchar=10)

            func = function('delete', ip_dest, next_hop,
                            next_hop_ip=next_hop_ip,
                            distance=distance,
                            tag=tag,
                            route_name=route_name)

            # Build the expected string for comparison
            # A value of None will default to an empty string, and
            # add the tag or name keywords where appropriate
            cmd_next_hop_ip = cmd_distance = cmd_tag = cmd_route_name = ''
            if next_hop_ip is not None:
                cmd_next_hop_ip = " %s" % next_hop_ip
            if distance is not None:
                cmd_distance = " %d" % distance
            if tag is not None:
                cmd_tag = " tag %d" % tag
            if route_name is not None:
                cmd_route_name = " name %s" % route_name
            cmds = "no ip route %s %s%s%s%s%s" % \
                   (ip_dest, next_hop, cmd_next_hop_ip, cmd_distance,
                    cmd_tag, cmd_route_name)

            self.eapi_positive_config_test(func, cmds)

    def test_default(self):
        # Test passing in a full set of parameters to 'default'
        # Some parameters may be not set: None
        for ip_dest in IP_DESTS:
            (next_hop, next_hop_ip) = choice(NEXT_HOPS)
            distance = choice(DISTANCES)
            if distance:
                distance = random_int(0, 255)
            tag = choice(TAGS)
            if tag:
                tag = random_int(0, 255)
            route_name = choice(ROUTE_NAMES)
            if route_name:
                route_name = random_string(minchar=4, maxchar=10)

            func = function('default', ip_dest, next_hop,
                            next_hop_ip=next_hop_ip,
                            distance=distance,
                            tag=tag,
                            route_name=route_name)

            # Build the expected string for comparison
            # A value of None will default to an empty string, and
            # add the tag or name keywords where appropriate
            cmd_next_hop_ip = cmd_distance = cmd_tag = cmd_route_name = ''
            if next_hop_ip is not None:
                cmd_next_hop_ip = " %s" % next_hop_ip
            if distance is not None:
                cmd_distance = " %d" % distance
            if tag is not None:
                cmd_tag = " tag %d" % tag
            if route_name is not None:
                cmd_route_name = " name %s" % route_name
            cmds = "default ip route %s %s%s%s%s%s" % \
                   (ip_dest, next_hop, cmd_next_hop_ip, cmd_distance,
                    cmd_tag, cmd_route_name)

            self.eapi_positive_config_test(func, cmds)

    def test_set_tag(self):
        # Test passing in a new tag to the set_tag function
        ip_dest = '1.2.3.0/24'
        next_hop = 'Ethernet1'
        next_hop_ip = '1.1.1.1'
        distance = 10
        tag = '99'

        func = function('set_tag', ip_dest, next_hop,
                        next_hop_ip=next_hop_ip,
                        distance=distance, tag=tag)

        cmd = "ip route %s %s %s %s tag %s" % \
            (ip_dest, next_hop, next_hop_ip, distance, tag)

        self.eapi_positive_config_test(func, cmd)

    def test_set_route_name(self):
        # Test passing in a new tag to the set_tag function
        ip_dest = '1.2.3.0/24'
        next_hop = 'Ethernet1'
        next_hop_ip = '1.1.1.1'
        distance = 10
        route_name = 'test99'

        func = function('set_route_name', ip_dest, next_hop,
                        next_hop_ip=next_hop_ip,
                        distance=distance, route_name=route_name)

        cmd = "ip route %s %s %s %s name %s" % \
            (ip_dest, next_hop, next_hop_ip, distance, route_name)

        self.eapi_positive_config_test(func, cmd)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))


from testlib import get_fixture, function, random_int, random_string
from testlib import EapiConfigUnitTest

import pyeapi.api.ipinterfaces


class TestApiIpinterfaces(EapiConfigUnitTest):

    INTERFACES = ['Ethernet1', 'Ethernet1/1', 'Vlan1234', 'Management1',
                  'Port-Channel1']

    def __init__(self, *args, **kwargs):
        super(TestApiIpinterfaces, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.ipinterfaces.instance(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_get(self):
        result = self.instance.get('Loopback0')
        values = dict(name='Loopback0', address='1.1.1.1/32', mtu=1500)
        self.assertEqual(result, values)

    def test_getall(self):
        result = self.instance.getall()
        self.assertIsInstance(result, dict)
        self.assertEqual(len(result), 3)

    def test_instance_functions(self):
        for intf in self.INTERFACES:
            for name in ['create', 'delete']:
                if name == 'create':
                    cmds = ['interface %s' % intf, 'no switchport']
                elif name == 'delete':
                    cmds = ['interface %s' % intf, 'no ip address',
                            'switchport']
                func = function(name, intf)
                self.eapi_positive_config_test(func, cmds)

    def test_set_address_with_value(self):
        for intf in self.INTERFACES:
            value = '1.2.3.4/5'
            cmds = ['interface %s' % intf, 'ip address 1.2.3.4/5']
            func = function('set_address', intf, value)
            self.eapi_positive_config_test(func, cmds)

    def test_set_address_with_no_value(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'no ip address']
            func = function('set_address', intf, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_address_with_default(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'default ip address']
            func = function('set_address', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_address_invalid_value_raises_value_error(self):
        for intf in self.INTERFACES:
            # func = function('set_address', intf, None)
            # self.eapi_exception_config_test(func, ValueError)
            # If command_builder fails because value is None, uncomment
            # above lines and remove below lines.
            cmds = ['interface %s' % intf, 'no ip address']
            func = function('set_address', intf, value=None)
            self.eapi_positive_config_test(func, cmds)

    def test_set_mtu_with_values(self):
        for intf in self.INTERFACES:
            for value in [68, 65535, random_int(68, 65535)]:
                cmds = ['interface %s' % intf, 'mtu %s' % value]
                func = function('set_mtu', intf, value)
                self.eapi_positive_config_test(func, cmds)

    def test_set_mtu_with_no_value(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'no mtu']
            func = function('set_mtu', intf, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_mtu_default(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'default mtu']
            func = function('set_mtu', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_mtu_invalid_value_raises_value_error(self):
        for intf in self.INTERFACES:
            for value in [67, 65536, 'a' + random_string()]:
                func = function('set_mtu', intf, value)
                self.eapi_exception_config_test(func, ValueError)
            for value in [None]:
                # If command_builder fails because value is None, put None
                # in the first loop to check for value error, and remove
                # this second loop
                cmds = ['interface %s' % intf, 'no mtu']
                func = function('set_mtu', intf, value)
                self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2016, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from random import randint
from systestlib import DutSystemTest


def clear_ospf_config(dut, pid=None):
    if pid is None:
        try:
            pid = int(dut.get_config(params="section ospf")[0].split()[2])
            dut.config(['no router ospf %d' % pid])
        except IndexError:
            '''No OSPF configured'''
            pass
    else:
        dut.config(['no router ospf %d' % pid])


class TestApiOspf(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            dut.config(["router ospf 1", "router-id 1.1.1.1",
                        "network 2.2.2.0/24 area 0", "redistribute bgp"])
            ospf_response = dut.api('ospf').get()
            config = dict(router_id="1.1.1.1", ospf_process_id=1,
                          vrf='default',
                          networks=[dict(netmask='24', network="2.2.2.0",
                                         area="0.0.0.0")],
                          redistributions=[dict(protocol="bgp")],
                          shutdown=False)
            self.assertEqual(ospf_response, config)

    def test_get_with_vrf(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            dut.config(["router ospf 10 vrf test", "router-id 1.1.1.2",
                        "network 2.2.2.0/24 area 0", "redistribute bgp"])
            ospf_response = dut.api('ospf').get()
            config = dict(router_id="1.1.1.2", ospf_process_id=10, vrf='test',
                          networks=[dict(netmask='24', network="2.2.2.0",
                                         area="0.0.0.0")],
                          redistributions=[dict(protocol="bgp")],
                          shutdown=False)
            self.assertEqual(ospf_response, config)
            clear_ospf_config(dut, 10)

    def test_shutdown(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            dut.config(["router ospf 1", "network 1.1.1.1/32 area 0"])
            ospf = dut.api('ospf')
            response = ospf.set_shutdown()
            self.assertTrue(response)
            self.assertIn('shutdown', ospf.get_block("router ospf 1"))

    def test_no_shutown(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            dut.config(["router ospf 10", "network 1.1.1.0/24 area 0",
                       "shutdown"])
            ospf = dut.api('ospf')
            response = ospf.set_no_shutdown()
            self.assertTrue(response)
            self.assertIn('no shutdown', ospf.get_block("router ospf 10"))

    def test_delete(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            dut.config(["router ospf 10"])
            ospf = dut.api("ospf")
            response = ospf.delete()
            self.assertTrue(response)
            self.assertEqual(None, ospf.get_block("router ospf"))

    def test_create_valid_id(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            pid = randint(1, 65536)
            ospf = dut.api("ospf")
            response = ospf.create(pid)
            self.assertTrue(response)
            self.assertIn("router ospf {}".format(pid), dut.get_config())

    def test_create_invalid_id(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            pid = randint(70000, 100000)
            with self.assertRaises(ValueError):
                dut.api("ospf").create(pid)

    def test_create_with_vrf(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            pid = randint(1, 65536)
            ospf = dut.api("ospf")
            response = ospf.create(pid, vrf='test')
            self.assertTrue(response)
            self.assertIn("router ospf {} vrf {}".format(pid, 'test'),
                          dut.get_config())

    def test_configure_ospf(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            dut.config(["router ospf 1"])
            ospf = dut.api("ospf")
            response = ospf.configure_ospf("router-id 1.1.1.1")
            self.assertTrue(response)
            self.assertIn("router-id 1.1.1.1", ospf.get_block("router ospf 1"))

    def test_set_router_id(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            dut.config(["router ospf 1"])
            ospf = dut.api("ospf")
            response = ospf.set_router_id(randint(1, 65536))
            self.assertFalse(response)
            response = ospf.set_router_id("2.2.2.2")
            self.assertTrue(response)
            self.assertIn("router-id 2.2.2.2", ospf.get_block("router ospf 1"))
            response = ospf.set_router_id(default=True)
            self.assertTrue(response)
            self.assertIn("no router-id", ospf.get_block("router ospf 1"))
            response = ospf.set_router_id(disable=True)
            self.assertTrue(response)
            self.assertIn("no router-id", ospf.get_block("router ospf 1"))

    def test_add_network(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            dut.config(["router ospf 1"])
            ospf = dut.api("ospf")
            response = ospf.add_network("2.2.2.0", "24", 1234)
            self.assertTrue(response)
            self.assertIn("network 2.2.2.0/24 area 0.0.4.210", ospf.get_block("router ospf 1"))
            response = ospf.add_network("10.10.10.0", "24")
            self.assertTrue(response)
            self.assertIn("network 10.10.10.0/24 area 0.0.0.0", ospf.get_block("router ospf 1"))

    def test_remove_network(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            ospf_config = ["router ospf 1", "network 2.2.2.0/24 area 0.0.0.0",
                           "network 3.3.3.1/32 area 1.1.1.1"]
            dut.config(ospf_config)
            ospf = dut.api("ospf")
            response = ospf.remove_network("2.2.2.0", "24")
            self.assertTrue(response)
            response = ospf.remove_network("3.3.3.1", "32", "1.1.1.1")
            self.assertTrue(response)
            for config in ospf_config:
                if "router ospf" not in config:
                    self.assertNotIn(config, ospf.get_block("router ospf 1"))

    def test_add_redistribution(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            dut.config(["router ospf 1"])
            ospf = dut.api("ospf")
            protos = ['bgp', 'rip', 'static', 'connected']
            for proto in protos:
                if randint(1, 10) % 2 == 0:
                    response = ospf.add_redistribution(proto, 'test')
                else:
                    response = ospf.add_redistribution(proto)
                self.assertTrue(response)
            for proto in protos:
                self.assertIn("redistribute {}".format(proto), ospf.get_block("router ospf 1"))
            with self.assertRaises(ValueError):
                ospf.add_redistribution("NOT VALID")

    def test_remove_redistribution(self):
        for dut in self.duts:
            clear_ospf_config(dut)
            dut.config(["router ospf 1", "redistribute bgp", "redistribute static route-map test"])
            ospf = dut.api("ospf")
            response = ospf.remove_redistribution('bgp')
            self.assertTrue(response)
            response = ospf.remove_redistribution('static')
            self.assertTrue(response)
            self.assertNotIn("redistribute", ospf.get_block("router ospf 1"))





#!/usr/bin/env python
from __future__ import print_function
import pyeapi

connection = pyeapi.connect(host='192.168.1.16')
output = connection.execute(['enable', 'show version'])

print(('My system MAC address is', output['result'][1]['systemMacAddress']))





#
# Copyright (c) 2017, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import random_string
from systestlib import DutSystemTest


class TestApiVrfs(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah',
                            'rd 10:10', 'description blah desc'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah',
                            'rd 10:10', 'description blah desc'])
            response = dut.api('vrfs').get('blah')
            values = dict(rd='10:10', vrf_name='blah', description='blah desc',
                          ipv4_routing=False, ipv6_routing=False)
            self.assertEqual(values, response)
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah'])
            else:
                dut.config(['no vrf definition blah'])

    def test_getall(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah',
                            'no vrf instance second', 'vrf instance second'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah',
                            'no vrf definition second', 'vrf definition second'])
            response = dut.api('vrfs').getall()
            self.assertIsInstance(response, dict, 'dut=%s' % dut)
            self.assertEqual(len(response), 2)
            for vrf_name in ['blah', 'second']:
                self.assertIn(vrf_name, response, 'dut=%s' % dut)
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'no vrf instance second'])
            else:
                dut.config(['no vrf definition blah', 'no vrf definition second'])

    def test_create_and_return_true(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah'])
            result = dut.api('vrfs').create('blah')
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vrf', encoding='text')
            self.assertIn('blah', config[0]['output'], 'dut=%s' % dut)
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah'])
            else:
                dut.config(['no vrf definition blah'])

    def test_create_with_valid_rd(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah'])
            result = dut.api('vrfs').create('blah', rd='10:10')
            self.assertTrue(result, 'dut=%s' % dut)
            command = 'show running-config section vrf'
            config = dut.run_commands(command, encoding='text')
            if dut.version_number >= '4.23':
                self.assertIn('vrf instance blah', config[0]['output'],
                              'dut=%s' % dut)
            else:
                self.assertIn('vrf definition blah', config[0]['output'],
                              'dut=%s' % dut)
            self.assertIn('rd 10:10', config[0]['output'], 'dut=%s' % dut)
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah'])
            else:
                dut.config(['no vrf definition blah'])

    def test_create_and_return_false(self):
        for dut in self.duts:
            result = dut.api('vrfs').create('a%')
            self.assertFalse(result, 'dut=%s' % dut)

    def test_create_with_invalid_rd(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah'])
            result = dut.api('vrfs').create('blah', rd='192.168.1.1:99999999')
            self.assertFalse(result, 'dut=%s' % dut)
            command = 'show running-config section vrf'
            config = dut.run_commands(command, encoding='text')
            if dut.version_number >= '4.23':
                self.assertIn('vrf instance blah', config[0]['output'],
                              'dut=%s' % dut)
            else:
                self.assertIn('vrf definition blah', config[0]['output'],
                              'dut=%s' % dut)
            self.assertNotIn('rd', config[0]['output'],
                             'dut=%s' % dut)
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah'])
            else:
                dut.config(['no vrf definition blah'])

    def test_delete_and_return_true(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config('vrf instance blah')
            else:
                dut.config('vrf definition blah')
            result = dut.api('vrfs').delete('blah')
            self.assertTrue(result, 'dut=%s' % dut)
            command = 'show running-config section vrf'
            config = dut.run_commands(command, encoding='text')
            if dut.version_number >= '4.23':
                self.assertNotIn('vrf instance blah', config[0]['output'],
                                 'dut=%s' % dut)
            else:
                self.assertNotIn('vrf definition blah', config[0]['output'],
                                 'dut=%s' % dut)

    def test_delete_and_return_false(self):
        for dut in self.duts:
            result = dut.api('vrfs').delete('a%')
            self.assertFalse(result, 'dut=%s' % dut)

    def test_default(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah',
                            'description test desc'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah',
                            'description test desc'])
            result = dut.api('vrfs').default('blah')
            self.assertTrue(result, 'dut=%s' % dut)
            command = 'show running-config section vrf'
            config = dut.run_commands(command, encoding='text')
            if dut.version_number >= '4.23':
                self.assertNotIn('vrf instance blah', config[0]['output'],
                                 'dut=%s' % dut)
                dut.config(['no vrf instance blah'])
            else:
                self.assertNotIn('vrf definition blah', config[0]['output'],
                                 'dut=%s' % dut)
                dut.config(['no vrf definition blah'])

    def test_set_rd(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah'])
            result = dut.api('vrfs').set_rd('blah', '10:10')
            self.assertTrue(result, 'dut=%s' % dut)
            command = 'show running-config section vrf'
            config = dut.run_commands(command, encoding='text')
            self.assertIn('blah', config[0]['output'], 'dut=%s' % dut)
            self.assertIn('10:10', config[0]['output'], 'dut=%s' % dut)
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah'])
            else:
                dut.config(['no vrf definition blah'])

    def test_set_description(self):
        for dut in self.duts:
            description = random_string()
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah'])
            result = dut.api('vrfs').set_description('blah', description)
            self.assertTrue(result, 'dut=%s' % dut)
            command = 'show running-config section vrf'
            config = dut.run_commands(command, encoding='text')
            self.assertIn('description %s' % description, config[0]['output'],
                          'dut=%s' % dut)
            result = dut.api('vrfs').set_description('blah', default=True)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands(command, encoding='text')
            self.assertNotIn('description %s' % description,
                             config[0]['output'], 'dut=%s' % dut)
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah'])
            else:
                dut.config(['no vrf definition blah'])

    def test_set_ipv4_routing(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah',
                            'rd 10:10', 'description test'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah',
                            'rd 10:10', 'description test'])
            result = dut.api('vrfs').set_ipv4_routing('blah')
            self.assertTrue(result, 'dut=%s' % dut)
            command = 'show running-config section vrf'
            config = dut.run_commands(command, encoding='text')
            self.assertIn('ip routing vrf blah', config[0]['output'],
                          'dut=%s' % dut)
            result = dut.api('vrfs').set_ipv4_routing('blah', default=True)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands(command, encoding='text')
            self.assertIn('no ip routing vrf blah', config[0]['output'],
                          'dut=%s' % dut)
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah'])
            else:
                dut.config(['no vrf definition blah'])

    def test_set_ipv6_routing(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah',
                            'rd 10:10', 'description test'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah',
                            'rd 10:10', 'description test'])
            result = dut.api('vrfs').set_ipv6_routing('blah')
            self.assertTrue(result, 'dut=%s' % dut)
            command = 'show running-config all section vrf'
            config = dut.run_commands(command, encoding='text')
            self.assertIn('ipv6 unicast-routing vrf blah', config[0]['output'],
                          'dut=%s' % dut)
            result = dut.api('vrfs').set_ipv6_routing('blah', default=True)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands(command, encoding='text')
            self.assertIn('no ipv6 unicast-routing vrf blah',
                          config[0]['output'], 'dut=%s' % dut)
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah'])
            else:
                dut.config(['no vrf definition blah'])

    def test_set_interface(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no vrf instance blah', 'vrf instance blah',
                            'rd 10:10', 'default interface Ethernet1',
                            'interface Ethernet1', 'no switchport'])
            else:
                dut.config(['no vrf definition blah', 'vrf definition blah',
                            'rd 10:10', 'default interface Ethernet1',
                            'interface Ethernet1', 'no switchport'])
            result = dut.api('vrfs').set_interface('blah', 'Ethernet1')
            self.assertTrue(result, 'dut=%s' % dut)
            command = 'show running-config interfaces Ethernet1'
            config = dut.run_commands(command, encoding='text')
            if dut.version_number >= '4.23':
                self.assertIn('vrf blah', config[0]['output'],
                              'dut=%s' % dut)
            else:
                self.assertIn('vrf forwarding blah', config[0]['output'],
                              'dut=%s' % dut)
            result = dut.api('vrfs').set_interface('blah', 'Ethernet1',
                                                   disable=True)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands(command, encoding='text')
            if dut.version_number >= '4.23':
                self.assertNotIn('vrf blah', config[0]['output'],
                                 'dut=%s' % dut)
                dut.config(['no vrf instance blah',
                            'default interface Ethernet1'])
            else:
                self.assertNotIn('vrf forwarding blah', config[0]['output'],
                                 'dut=%s' % dut)
                dut.config(['no vrf definition blah',
                            'default interface Ethernet1'])


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import random
import string
import unittest

from mock import MagicMock as Mock

from pyeapi.client import Node

def get_fixtures_path():
    return os.path.join(os.path.dirname(__file__), '../fixtures')

def get_fixture(filename):
    return os.path.join(get_fixtures_path(), filename)

def random_string(minchar=1, maxchar=50):
    return ''.join(random.choice(string.ascii_uppercase + string.digits)
                   for _ in range(random.randint(minchar, maxchar)))

def random_vlan():
    return random.randint(1, 4094)

def random_int(minvalue, maxvalue):
    return random.randint(minvalue, maxvalue)


from collections import namedtuple
Function = namedtuple('Function', 'name args kwargs')

def function(name, *args, **kwargs):
    return Function(name, args, kwargs)


class EapiConfigUnitTest(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        self.instance = None
        self.config = None
        super(EapiConfigUnitTest, self).__init__(*args, **kwargs)

    def setUp(self):
        self.node = Node(None)
        self.node._version_number = '4.17.1.1'
        self.node._running_config = self.config

        self.mock_config = Mock(name='node.config')
        self.node.config = self.mock_config

        self.mock_enable = Mock(name='node.enable')
        self.node.enable = self.mock_enable

        self.assertIsNotNone(self.instance)
        self.instance.node = self.node

    def eapi_config_test(self, func, cmds=None, *args, **kwargs):
        func, fargs, fkwargs = func
        func = getattr(self.instance, func)

        if cmds is not None:
            lcmds = len([cmds]) if isinstance(cmds, str) else len(cmds)
            self.mock_config.return_value = [{} for i in range(0, lcmds)]

        result = func(*fargs, **fkwargs)

        if cmds is not None:
            self.node.config.assert_called_with(cmds)
        else:
            self.assertEqual(self.node.config.call_count, 0)

        return result

    def eapi_positive_config_test(self, func, cmds=None, *args, **kwargs):
        result = self.eapi_config_test(func, cmds, *args, **kwargs)
        self.assertTrue(result)

    def eapi_negative_config_test(self, func, cmds=None, *args, **kwargs):
        result = self.eapi_config_test(func, cmds, *args, **kwargs)
        self.assertFalse(result)

    def eapi_exception_config_test(self, func, exc, *args, **kwargs):
        with self.assertRaises(exc):
            self.eapi_config_test(func, *args, **kwargs)

    def eapi_positive_config_with_input_test(self, func, cmds=None,
                                             *args, **kwargs):
        result = self.eapi_config_test(func, cmds, *args, **kwargs)
        self.assertTrue(result)



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from systestlib import DutSystemTest
from testlib import random_string


class TestApiRoutemaps(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'set weight 100',
                        'match tag 50'])
            response = dut.api('routemaps').get('TEST')
            self.assertIsNotNone(response)

    def test_get_none(self):
        for dut in self.duts:
            dut.config('no route-map TEST deny 10')
            result = dut.api('routemaps').get('TEST')
            self.assertIsNone(result)

    def test_getall(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'set weight 100',
                        'no route-map TEST-2 permit 50',
                        'route-map TEST-2 permit 50',
                        'match tag 50'])
            result = dut.api('routemaps').getall()
            self.assertIn(('TEST'), result)
            self.assertIn(('TEST-2'), result)

    def test_create(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10'])
            api = dut.api('routemaps')
            self.assertIsNone(api.get('TEST'))
            result = dut.api('routemaps').create('TEST', 'deny', 10)
            self.assertTrue(result)
            self.assertIsNotNone(api.get('TEST'))
            dut.config(['no route-map TEST deny 10'])

    def test_create_with_hyphen(self):
        for dut in self.duts:
            dut.config(['no route-map TEST-1 deny 10'])
            api = dut.api('routemaps')
            self.assertIsNone(api.get('TEST-1'))
            result = dut.api('routemaps').create('TEST-1', 'deny', 10)
            self.assertTrue(result)
            self.assertIsNotNone(api.get('TEST-1'))
            dut.config(['no route-map TEST-1 deny 10'])

    def test_create_with_underscore(self):
        for dut in self.duts:
            dut.config(['no route-map TEST_1 deny 10'])
            api = dut.api('routemaps')
            self.assertIsNone(api.get('TEST_1'))
            result = dut.api('routemaps').create('TEST_1', 'deny', 10)
            self.assertTrue(result)
            self.assertIsNotNone(api.get('TEST_1'))
            dut.config(['no route-map TEST_1 deny 10'])

    def test_delete(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'set weight 100'])
            api = dut.api('routemaps')
            self.assertIsNotNone(api.get('TEST'))
            result = dut.api('routemaps').delete('TEST', 'deny', 10)
            self.assertTrue(result)
            self.assertIsNone(api.get('TEST'))

    def test_default(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'set weight 100'])
            api = dut.api('routemaps')
            self.assertIsNotNone(api.get('TEST'))
            result = dut.api('routemaps').default('TEST', 'deny', 10)
            self.assertTrue(result)
            self.assertIsNone(api.get('TEST'))

    def test_set_description(self):
        for dut in self.duts:
            text = random_string()
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10'])
            api = dut.api('routemaps')
            self.assertNotIn('description %s' % text,
                             api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_description('TEST', 'deny', 10,
                                                          text)
            self.assertTrue(result)
            self.assertIn('description %s' % text,
                          api.get_block('route-map TEST deny 10'))

    def test_set_match_statements(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10'])
            api = dut.api('routemaps')
            self.assertNotIn('match as 100',
                             api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_match_statements('TEST', 'deny',
                                                               10, ['as 100'])
            self.assertTrue(result)
            self.assertIn('match as 100',
                          api.get_block('route-map TEST deny 10'))

    def test_update_match_statement(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'match as 100'])
            api = dut.api('routemaps')
            self.assertIn('match as 100',
                          api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_match_statements('TEST', 'deny',
                                                               10, ['as 200'])
            self.assertTrue(result)
            self.assertNotIn('match as 100',
                             api.get_block('route-map TEST deny 10'))
            self.assertIn('match as 200',
                          api.get_block('route-map TEST deny 10'))

    def test_remove_match_statement(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'match as 100'])
            api = dut.api('routemaps')
            self.assertIn('match as 100',
                          api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_match_statements('TEST', 'deny',
                                                               10, ['tag 50'])
            self.assertTrue(result)
            self.assertNotIn('match as 100',
                             api.get_block('route-map TEST deny 10'))
            self.assertIn('match tag 50',
                          api.get_block('route-map TEST deny 10'))

    def test_set_set_statements(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10'])
            api = dut.api('routemaps')
            self.assertNotIn('set weight 100',
                             api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_set_statements('TEST', 'deny',
                                                             10, ['weight 100'])
            self.assertTrue(result)
            self.assertIn('set weight 100',
                          api.get_block('route-map TEST deny 10'))

    def test_update_set_statement(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'set weight 100'])
            api = dut.api('routemaps')
            self.assertIn('set weight 100',
                          api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_set_statements('TEST', 'deny',
                                                             10, ['weight 200'])
            self.assertTrue(result)
            self.assertNotIn('set weight 100',
                             api.get_block('route-map TEST deny 10'))
            self.assertIn('set weight 200',
                          api.get_block('route-map TEST deny 10'))

    def test_remove_set_statement(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'set weight 100'])
            api = dut.api('routemaps')
            self.assertIn('set weight 100',
                          api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_set_statements('TEST', 'deny',
                                                             10, ['tag 50'])
            self.assertTrue(result)
            self.assertNotIn('set weight 100',
                             api.get_block('route-map TEST deny 10'))
            self.assertIn('set tag 50',
                          api.get_block('route-map TEST deny 10'))

    def test_set_continue(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10'])
            api = dut.api('routemaps')
            self.assertNotIn('continue',
                             api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_continue('TEST', 'deny', 10, 100)
            self.assertTrue(result)
            self.assertEqual(100, api.get('TEST')['deny'][10]['continue'])

    def test_update_continue(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'continue 30'])
            api = dut.api('routemaps')
            self.assertIn('continue 30',
                          api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_continue('TEST', 'deny', 10, 60)
            self.assertTrue(result)
            self.assertEqual(60, api.get('TEST')['deny'][10]['continue'])

    def test_default_continue(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'continue 100'])
            api = dut.api('routemaps')
            self.assertIn('continue 100',
                          api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_continue('TEST', 'deny', 10,
                                                       default=True)
            self.assertTrue(result)
            self.assertEqual(None, api.get('TEST')['deny'][10]['continue'])

    def test_negate_continue(self):
        for dut in self.duts:
            dut.config(['no route-map TEST deny 10',
                        'route-map TEST deny 10',
                        'continue 100'])
            api = dut.api('routemaps')
            self.assertIn('continue 100',
                          api.get_block('route-map TEST deny 10'))
            result = dut.api('routemaps').set_continue('TEST', 'deny', 10,
                                                       disable=True)
            self.assertTrue(result)
            self.assertEqual(None, api.get('TEST')['deny'][10]['continue'])


if __name__ == '__main__':
    unittest.main()



import unittest
import json

from mock import Mock, patch

import pyeapi.eapilib


class TestEapiConnection(unittest.TestCase):

    def test_execute_valid_response(self):
        response_dict = dict(jsonrpc='2.0', result=[], id=id(self))
        mock_send = Mock(name='send')
        mock_send.return_value = json.dumps(response_dict)

        instance = pyeapi.eapilib.EapiConnection()
        instance.send = mock_send

        result = instance.execute(['command'])
        self.assertEqual(json.loads(result), response_dict)

    def test_execute_raises_type_error(self):
        instance = pyeapi.eapilib.EapiConnection()
        with self.assertRaises(TypeError):
            instance.execute(None, encoding='invalid')

    def test_execute_raises_connection_error(self):
        mock_send = Mock(name='send')
        mock_send.side_effect = pyeapi.eapilib.ConnectionError('test', 'test')

        instance = pyeapi.eapilib.EapiConnection()
        instance.send = mock_send

        with self.assertRaises(pyeapi.eapilib.ConnectionError):
            instance.execute('test')

    def test_execute_raises_command_error(self):
        mock_send = Mock(name='send')
        mock_send.side_effect = pyeapi.eapilib.CommandError('1000', 'test')

        instance = pyeapi.eapilib.EapiConnection()
        instance.send = mock_send

        with self.assertRaises(pyeapi.eapilib.CommandError):
            instance.execute('test')

    def test_create_socket_connection(self):
        instance = pyeapi.eapilib.SocketEapiConnection()
        self.assertIsInstance(instance, pyeapi.eapilib.EapiConnection)
        self.assertIsNotNone(str(instance.transport))

    @patch('pyeapi.eapilib.socket')
    def test_socket_connection_create(self, mock_socket):
        instance = pyeapi.eapilib.SocketConnection('/path/to/sock')
        instance.connect()
        mock_socket.socket.return_value.connect.assert_called_with('/path/to/sock')

    def test_create_http_local_connection(self):
        instance = pyeapi.eapilib.HttpLocalEapiConnection()
        self.assertIsInstance(instance, pyeapi.eapilib.EapiConnection)
        self.assertIsNotNone(str(instance.transport))

    def test_create_http_connection(self):
        instance = pyeapi.eapilib.HttpEapiConnection('localhost')
        self.assertIsInstance(instance, pyeapi.eapilib.EapiConnection)
        self.assertIsNotNone(str(instance.transport))

    def test_create_https_connection(self):
        instance = pyeapi.eapilib.HttpsEapiConnection('localhost')
        self.assertIsInstance(instance, pyeapi.eapilib.EapiConnection)
        self.assertIsNotNone(str(instance.transport))

    def test_send(self):
        response_dict = dict(jsonrpc='2.0', result=[{}], id=id(self))
        response_json = json.dumps(response_dict)

        mock_transport = Mock(name='transport')
        mockcfg = {'getresponse.return_value.read.return_value': response_json}
        mock_transport.configure_mock(**mockcfg)

        instance = pyeapi.eapilib.EapiConnection()
        instance.transport = mock_transport
        instance.send('test')
        # HTTP requests to be processed by EAPI should always go to
        # the /command-api endpoint regardless of using TCP/IP or unix-socket
        # for the transport. Unix-socket implementation maps localhost to the
        # unix-socket - /var/run/command-api.sock
        mock_transport.putrequest.assert_called_once_with('POST',
                                                          '/command-api')
        self.assertTrue(mock_transport.close.called)

    def test_send_with_authentication(self):
        response_dict = dict(jsonrpc='2.0', result=[{}], id=id(self))
        response_json = json.dumps(response_dict)

        mock_transport = Mock(name='transport')
        mockcfg = {'getresponse.return_value.read.return_value': response_json}
        mock_transport.configure_mock(**mockcfg)

        instance = pyeapi.eapilib.EapiConnection()
        instance.authentication('username', 'password')
        instance.transport = mock_transport
        instance.send('test')

        self.assertTrue(mock_transport.close.called)

    def test_send_unauthorized_user(self):
        error_string = ('Unauthorized. Unable to authenticate user: Bad'
                        ' username/password combination')
        response_str = ('Unable to authenticate user: Bad username/password'
                        ' combination')
        mock_transport = Mock(name='transport')
        mockcfg = {'getresponse.return_value.read.return_value': response_str,
                   'getresponse.return_value.status': 401,
                   'getresponse.return_value.reason': 'Unauthorized'}
        mock_transport.configure_mock(**mockcfg)

        instance = pyeapi.eapilib.EapiConnection()
        instance.authentication('username', 'password')
        instance.transport = mock_transport
        try:
            instance.send('test')
        except pyeapi.eapilib.ConnectionError as err:
            self.assertEqual(err.message, error_string)

    def test_send_raises_connection_error(self):
        mock_transport = Mock(name='transport')
        mockcfg = {'getresponse.return_value.read.side_effect': ValueError}
        mock_transport.configure_mock(**mockcfg)

        instance = pyeapi.eapilib.EapiConnection()
        instance.transport = mock_transport
        try:
            instance.send('test')
        except pyeapi.eapilib.ConnectionError as err:
            self.assertEqual(err.message, 'unable to connect to eAPI')

    def test_send_raises_connection_socket_error(self):
        mock_transport = Mock(name='transport')
        mockcfg = {'getresponse.return_value.read.side_effect':
                   OSError('timeout')}
        mock_transport.configure_mock(**mockcfg)

        instance = pyeapi.eapilib.EapiConnection()
        instance.transport = mock_transport
        try:
            instance.send('test')
        except pyeapi.eapilib.ConnectionError as err:
            error_msg = 'Socket error during eAPI connection: timeout'
            self.assertEqual(err.message, error_msg)

    def test_send_raises_command_error(self):
        error = dict(code=9999, message='test', data=[{'errors': ['test']}])
        response_dict = dict(jsonrpc='2.0', error=error, id=id(self))
        response_json = json.dumps(response_dict)

        mock_transport = Mock(name='transport')
        mockcfg = {'getresponse.return_value.read.return_value': response_json}
        mock_transport.configure_mock(**mockcfg)

        instance = pyeapi.eapilib.EapiConnection()
        instance.transport = mock_transport

        with self.assertRaises(pyeapi.eapilib.CommandError):
            instance.send('test')

    def test_send_raises_autocomplete_command_error(self):
        message = "runCmds() got an unexpected keyword argument 'autoComplete'"
        error = dict(code=9999, message=message, data=[{'errors': ['test']}])
        response_dict = dict(jsonrpc='2.0', error=error, id=id(self))
        response_json = json.dumps(response_dict)

        mock_transport = Mock(name='transport')
        mockcfg = {'getresponse.return_value.read.return_value': response_json}
        mock_transport.configure_mock(**mockcfg)

        instance = pyeapi.eapilib.EapiConnection()
        instance.transport = mock_transport

        try:
            instance.send('test')
        except pyeapi.eapilib.CommandError as error:
            match = ("autoComplete parameter is not supported in this version"
                     " of EOS.")
            self.assertIn(match, error.message)

    def test_send_raises_expandaliases_command_error(self):
        message = "runCmds() got an unexpected keyword argument" \
                  " 'expandAliases'"
        error = dict(code=9999, message=message, data=[{'errors': ['test']}])
        response_dict = dict(jsonrpc='2.0', error=error, id=id(self))
        response_json = json.dumps(response_dict)

        mock_transport = Mock(name='transport')
        mockcfg = {'getresponse.return_value.read.return_value': response_json}
        mock_transport.configure_mock(**mockcfg)

        instance = pyeapi.eapilib.EapiConnection()
        instance.transport = mock_transport

        try:
            instance.send('test')
        except pyeapi.eapilib.CommandError as error:
            match = ("expandAliases parameter is not supported in this version"
                     " of EOS.")
            self.assertIn(match, error.message)

    def test_request_adds_autocomplete(self):
        instance = pyeapi.eapilib.EapiConnection()
        request = instance.request(['sh ver'], encoding='json',
                                   autoComplete=True)
        data = json.loads(request)
        self.assertIn('autoComplete', data['params'])

    def test_request_adds_expandaliases(self):
        instance = pyeapi.eapilib.EapiConnection()
        request = instance.request(['test'], encoding='json',
                                   expandAliases=True)
        data = json.loads(request)
        self.assertIn('expandAliases', data['params'])

    def test_request_ignores_unknown_param(self):
        instance = pyeapi.eapilib.EapiConnection()
        request = instance.request(['sh ver'], encoding='json',
                                   unknown=True)
        data = json.loads(request)
        self.assertNotIn('unknown', data['params'])


class TestCommandError(unittest.TestCase):

    def test_create_command_error(self):
        result = pyeapi.eapilib.CommandError(9999, 'test')
        self.assertIsInstance(result, pyeapi.eapilib.EapiError)

    def test_command_error_trace(self):
        commands = ['test command', 'test command', 'test command']
        output = [{}, 'test output']
        result = pyeapi.eapilib.CommandError(9999, 'test', commands=commands,
                                             output=output)
        self.assertIsNotNone(result.trace)



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from systestlib import DutSystemTest

TEST_SSH_KEY = ('ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDKL1UtBALa4CvFUsHUipNym'
                'A04qCXuAtTwNcMj84bTUzUI+q7mdzRCTLkllXeVxKuBnaTm2PW7W67K5CVpl0'
                'EVCm6IY7FS7kc4nlnD/tFvTvShy/fzYQRAdM7ZfVtegW8sMSFJzBR/T/Y/sxI'
                '16Y/dQb8fC3la9T25XOrzsFrQiKRZmJGwg8d+0RLxpfMg0s/9ATwQKp6tPoLE'
                '4f3dKlAgSk5eENyVLA3RsypWADHpenHPcB7sa8D38e1TS+n+EUyAdb3Yov+5E'
                'SAbgLIJLd52Xv+FyYi0c2L49ByBjcRrupp4zfXn4DNRnEG4K6GcmswHuMEGZv'
                '5vjJ9OYaaaaaaa')

class TestApiUsers(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no username test', 'username test nopassword',
                            'username test ssh-key %s' % TEST_SSH_KEY])
            else:
                dut.config(['no username test', 'username test nopassword',
                            'username test sshkey %s' % TEST_SSH_KEY])

            result = dut.api('users').get('test')
            if dut.version_number >= '4.23':
                values = dict(nopassword=True, privilege='1', secret='',
                              role='', format='')
                values["ssh-key"] = TEST_SSH_KEY
            else:
                values = dict(nopassword=True, privilege='1', secret='',
                              role='', format='', sshkey=TEST_SSH_KEY)
            result = self.sort_dict_by_keys(result)
            values = self.sort_dict_by_keys(values)

            self.assertEqual(values, result)

    def test_getall(self):
        for dut in self.duts:
            dut.config(['no username test', 'username test nopassword'])
            result = dut.api('users').getall()
            self.assertIsInstance(result, dict, 'dut=%s' % dut)
            self.assertIn('test', result, 'dut=%s' % dut)

    def test_create(self):
        for dut in self.duts:
            dut.config(['no username test', 'username test nopassword'])
            api = dut.api('users')
            result = api.create('test', nopassword=True)
            self.assertTrue(result)
            self.assertIn('username test privilege 1', api.config)

    def test_delete(self):
        for dut in self.duts:
            dut.config(['username test nopassword'])
            api = dut.api('users')
            self.assertIn('username test privilege 1 nopassword', api.config)
            result = api.delete('test')
            self.assertTrue(result)
            self.assertNotIn('username test privilege 1 nopassword', api.config)

    def test_default(self):
        for dut in self.duts:
            dut.config(['username test nopassword'])
            api = dut.api('users')
            self.assertIn('username test privilege 1 nopassword', api.config)
            result = api.default('test')
            self.assertTrue(result)
            self.assertNotIn('username test nopassword', api.config)

    def test_set_privilege_with_value(self):
        for dut in self.duts:
            dut.config(['no username test', 'username test nopassword'])
            api = dut.api('users')
            # EOS defaults to privilege 1
            self.assertIn('username test privilege 1 nopassword', api.config)
            result = api.set_privilege('test', 8)
            self.assertTrue(result)
            self.assertIn('username test privilege 8 nopassword', api.config)

    def test_set_privilege_with_no_value(self):
        for dut in self.duts:
            dut.config(['no username test',
                        'username test privilege 8 nopassword'])
            api = dut.api('users')
            self.assertIn('username test privilege 8', api.config)
            result = api.set_privilege('test')
            self.assertTrue(result)
            self.assertIn('username test privilege 1', api.config)

    def test_set_role_with_value(self):
        for dut in self.duts:
            dut.config(['no username test', 'username test nopassword'])
            api = dut.api('users')
            self.assertIn('username test privilege 1 nopassword', api.config)
            result = api.set_role('test', 'network-admin')
            self.assertTrue(result)
            self.assertIn('username test privilege 1 role network-admin nopassword', api.config)

    def test_set_role_with_no_value(self):
        for dut in self.duts:
            dut.config(['no username test',
                        'username test role network-admin nopassword'])
            api = dut.api('users')
            self.assertIn('username test privilege 1 role network-admin nopassword', api.config)
            result = api.set_role('test')
            self.assertTrue(result)
            self.assertNotIn('username test privilege 1 role network-admin nopassword', api.config)

    def test_set_sshkey_with_value(self):
        for dut in self.duts:
            dut.config(['no username test', 'username test nopassword'])
            api = dut.api('users')
            self.assertIn('username test privilege 1 nopassword', api.config)
            if dut.version_number >= '4.23':
                self.assertNotIn('username test ssh-key', api.config)
            else:
                self.assertNotIn('username test sshkey', api.config)
            result = api.set_sshkey('test', TEST_SSH_KEY)
            self.assertTrue(result)
            if dut.version_number >= '4.23':
                self.assertIn('username test ssh-key %s' % TEST_SSH_KEY, api.config)
            else:
                self.assertIn('username test sshkey %s' % TEST_SSH_KEY, api.config)

    def test_set_sshkey_with_empty_string(self):
        for dut in self.duts:
            dut.config(['no username test', 'username test nopassword'])
            api = dut.api('users')
            self.assertIn('username test privilege 1 nopassword', api.config)
            if dut.version_number >= '4.23':
                self.assertNotIn('username test ssh-key', api.config)
            else:
                self.assertNotIn('username test sshkey', api.config)
            result = api.set_sshkey('test', '')
            self.assertTrue(result)
            if dut.version_number >= '4.23':
                self.assertNotIn('username test ssh-key %s'
                                 % TEST_SSH_KEY, api.config)
            else:
                self.assertNotIn('username test sshkey %s'
                                 % TEST_SSH_KEY, api.config)

    def test_set_sshkey_with_None(self):
        for dut in self.duts:
            dut.config(['no username test', 'username test nopassword'])
            api = dut.api('users')
            self.assertIn('username test privilege 1 nopassword', api.config)
            if dut.version_number >= '4.23':
                self.assertNotIn('username test ssh-key', api.config)
            else:
                self.assertNotIn('username test sshkey', api.config)
            result = api.set_sshkey('test', None)
            self.assertTrue(result)
            if dut.version_number >= '4.23':
                self.assertNotIn('username test ssh-key %s'
                                 % TEST_SSH_KEY, api.config)
            else:
                self.assertNotIn('username test sshkey %s'
                                 % TEST_SSH_KEY, api.config)

    def test_set_sshkey_with_no_value(self):
        for dut in self.duts:
            dut.config(['no username test',
                        'username test nopassword'])
            api = dut.api('users')
            self.assertIn('username test privilege 1 nopassword', api.config)
            result = api.set_sshkey('test', disable=True)
            self.assertTrue(result)
            if dut.version_number >= '4.23':
                self.assertNotIn('username test ssh-key %s' % TEST_SSH_KEY,
                                 api.config)
            else:
                self.assertNotIn('username test sshkey %s' % TEST_SSH_KEY,
                                 api.config)


if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages
from codecs import open
from os import path, environ
import sys

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'docs/description.rst'), encoding='utf-8') as f:
        long_description = f.read()

with open(path.join(here, 'VERSION'), mode='r', encoding='utf-8') as version_file:
        version = version_file.read().strip()

setup(
    name='pyeapi',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version,

    description='Python Client for eAPI',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/arista-eosplus/pyeapi',

    # Author details
    author='Arista EOS+ CS',
    author_email='eosplus-dev@arista.com',

    # Choose your license
    license='BSD-3',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        'Intended Audience :: System Administrators',
        'Topic :: System :: Networking',

        'License :: OSI Approved :: BSD License',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],

    keywords='networking arista eos eapi',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # List run-time dependencies here.  These will be installed by pip when your
    # project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['netaddr'],

    # List additional dependencies here (e.g. development dependencies).
    # You can install these using the following syntax, for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': ['check-manifest', 'pep8', 'pyflakes', 'twine'],
        'test': ['coverage', 'mock'],
    },
)

def install():
    if "install" in sys.argv:
        return True
    else:
        return False

# Use the following to dynamically build pyeapi module documentation
if install() and environ.get('READTHEDOCS'):
    print('This method is only called by READTHEDOCS.')
    from subprocess import Popen
    proc = Popen(['make', 'modules'], cwd='docs/')
    (_, err) = proc.communicate()
    return_code = proc.wait()

    if return_code or err:
        raise ('Failed to make modules.(%s:%s)' % (return_code, err))



#!/usr/bin/env python
from __future__ import print_function
import pyeapi

pyeapi.load_config('nodes.conf')
node = pyeapi.connect_to('veos01')

output = node.enable('show version')

print(('My System MAC address is', output[0]['result']['systemMacAddress']))



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from systestlib import DutSystemTest


class TestApiStandardAcls(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            dut.config(['no ip access-list standard test',
                        'ip access-list standard test'])
            response = dut.api('acl').get('test')
            self.assertIsNotNone(response)

    def test_get_none(self):
        for dut in self.duts:
            dut.config('no ip access-list standard test')
            result = dut.api('acl').get('test')
            self.assertIsNone(result)

    def test_getall(self):
        for dut in self.duts:
            dut.config(['no ip access-list standard test',
                        'ip access-list standard test'])
            result = dut.api('acl').getall()
            self.assertIn('test', result['standard'])

    def test_create(self):
        for dut in self.duts:
            dut.config('no ip access-list standard test')
            api = dut.api('acl')
            self.assertIsNone(api.get('test'))
            result = dut.api('acl').create('test')
            self.assertTrue(result)
            self.assertIsNotNone(api.get('test'))

    def test_delete(self):
        for dut in self.duts:
            dut.config('ip access-list standard test')
            api = dut.api('acl')
            self.assertIsNotNone(api.get('test'))
            result = dut.api('acl').delete('test')
            self.assertTrue(result)
            self.assertIsNone(api.get('test'))

    def test_default(self):
        for dut in self.duts:
            dut.config('ip access-list standard test')
            api = dut.api('acl')
            self.assertIsNotNone(api.get('test'))
            result = dut.api('acl').default('test')
            self.assertTrue(result)
            self.assertIsNone(api.get('test'))

    def test_update_entry(self):
        for dut in self.duts:
            dut.config(['no ip access-list standard test',
                        'ip access-list standard test'])
            api = dut.api('acl')
            self.assertNotIn('10 permit any log',
                             api.get_block('ip access-list standard test'))
            result = dut.api('acl').update_entry('test', '10', 'permit',
                                                 '0.0.0.0', '0', True)
            self.assertTrue(result)
            self.assertIn('10 permit any log',
                          api.get_block('ip access-list standard test'))

    def test_update_entry_existing(self):
        for dut in self.duts:
            dut.config(['no ip access-list standard test',
                        'ip access-list standard test', '10 permit any log'])
            api = dut.api('acl')
            self.assertIn('10 permit any log',
                          api.get_block('ip access-list standard test'))
            result = dut.api('acl').update_entry('test', '10', 'deny',
                                                 '0.0.0.0', '0', True)
            self.assertTrue(result)
            self.assertIn('10 deny any log',
                          api.get_block('ip access-list standard test'))

    def test_add_entry(self):
        for dut in self.duts:
            dut.config(['no ip access-list standard test',
                        'ip access-list standard test'])
            api = dut.api('acl')
            self.assertNotIn('10 permit any log',
                             api.get_block('ip access-list standard test'))
            result = api.add_entry('test', 'permit', '0.0.0.0', '0', True)
            self.assertTrue(result)
            self.assertIn('10 permit any log',
                          api.get_block('ip access-list standard test'))

    def test_remove_entry(self):
        for dut in self.duts:
            dut.config(['no ip access-list standard test',
                        'ip access-list standard test', '10 permit any log'])
            api = dut.api('acl')
            self.assertIn('10 permit any log',
                          api.get_block('ip access-list standard test'))
            result = api.remove_entry('test', '10')
            self.assertTrue(result)
            self.assertNotIn('10 permit any log',
                             api.get_block('ip access-list standard test'))


class TestApiExtendedAcls(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            dut.config(['no ip access-list exttest',
                        'ip access-list exttest'])
            response = dut.api('acl').get('exttest')
            self.assertIsNotNone(response)

    def test_get_none(self):
        for dut in self.duts:
            dut.config('no ip access-list exttest')
            result = dut.api('acl').get('exttest')
            self.assertIsNone(result)

    def test_getall(self):
        for dut in self.duts:
            dut.config(['no ip access-list exttest',
                        'ip access-list exttest'])
            result = dut.api('acl').getall()
            self.assertIn('exttest', result['extended'])

    def test_create(self):
        for dut in self.duts:
            dut.config('no ip access-list exttest')
            api = dut.api('acl')
            self.assertIsNone(api.get('exttest'))
            result = dut.api('acl').create('exttest', 'extended')
            self.assertTrue(result)
            self.assertIsNotNone(api.get('exttest'))

    def test_delete(self):
        for dut in self.duts:
            dut.config('ip access-list exttest')
            api = dut.api('acl')
            self.assertIsNotNone(api.get('exttest'))
            result = dut.api('acl').delete('exttest')
            self.assertTrue(result)
            self.assertIsNone(api.get('exttest'))

    def test_default(self):
        for dut in self.duts:
            dut.config('ip access-list exttest')
            api = dut.api('acl')
            self.assertIsNotNone(api.get('exttest'))
            result = dut.api('acl').default('exttest')
            self.assertTrue(result)
            self.assertIsNone(api.get('exttest'))

    def test_update_entry(self):
        for dut in self.duts:
            dut.config(['no ip access-list exttest',
                        'ip access-list exttest'])
            api = dut.api('acl')
            self.assertNotIn('10 permit ip any any',
                             api.get_block('ip access-list exttest'))
            result = dut.api('acl').update_entry('exttest', '10', 'permit',
                                                 'ip', '0.0.0.0', '0',
                                                 '0.0.0.0', '0', False)
            self.assertTrue(result)
            self.assertIn('10 permit ip any any',
                          api.get_block('ip access-list exttest'))

    def test_update_entry_existing(self):
        for dut in self.duts:
            dut.config(['no ip access-list exttest',
                        'ip access-list exttest', '10 permit ip any any log'])
            api = dut.api('acl')
            self.assertIn('10 permit ip any any log',
                          api.get_block('ip access-list exttest'))
            result = dut.api('acl').update_entry('exttest', '10', 'deny', 'ip',
                                                 '0.0.0.0', '0', '0.0.0.0',
                                                 '0', True)
            self.assertTrue(result)
            self.assertIn('10 deny ip any any log',
                          api.get_block('ip access-list exttest'))

    def test_add_entry(self):
        for dut in self.duts:
            dut.config(['no ip access-list exttest',
                        'ip access-list exttest'])
            api = dut.api('acl')
            self.assertNotIn('10 permit ip any any log',
                             api.get_block('ip access-list exttest'))
            result = api.add_entry('exttest', 'permit', 'ip', '0.0.0.0', '0',
                                   '0.0.0.0', '0', True)
            self.assertTrue(result)
            self.assertIn('10 permit ip any any log',
                          api.get_block('ip access-list exttest'))

    def test_remove_entry(self):
        for dut in self.duts:
            dut.config(['no ip access-list exttest',
                        'ip access-list exttest', '10 permit ip any any log'])
            api = dut.api('acl')
            self.assertIn('10 permit ip any any log',
                          api.get_block('ip access-list exttest'))
            result = api.remove_entry('exttest', '10')
            self.assertTrue(result)
            self.assertNotIn('10 permit ip any any log',
                             api.get_block('ip access-list exttest'))


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, function
from testlib import EapiConfigUnitTest

import pyeapi.api.varp


class TestApiVarp(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiVarp, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.varp.Varp(None)
        self.config = open(get_fixture('running_config.varp')).read()

    def test_instance(self):
        result = pyeapi.api.varp.instance(None)
        self.assertIsInstance(result, pyeapi.api.varp.Varp)

    def test_get(self):
        result = self.instance.get()
        keys = ['mac_address', 'interfaces']
        self.assertEqual(sorted(keys), sorted(result.keys()))
        self.assertIsNotNone(self.instance.get()['mac_address'])
        self.assertIsNotNone(self.instance.get()['interfaces'])

    def test_get_interfaces_none(self):
        self._interfaces = None
        result = self.instance.interfaces()
        self.assertIsNotNone(result)

    def test_get_interfaces_already_defined(self):
        self.instance.interfaces()
        result = self.instance.interfaces()
        self.assertIsNotNone(result)

    def test_set_mac_address_with_value(self):
        value = 'aa:bb:cc:dd:ee:ff'
        func = function('set_mac_address', mac_address=value)
        cmds = 'ip virtual-router mac-address %s' % value
        self.eapi_positive_config_test(func, cmds)

    def test_set_mac_address_with_positional_value(self):
        value = 'aa:bb:cc:dd:ee:ff'
        func = function('set_mac_address', value)
        cmds = 'ip virtual-router mac-address %s' % value
        self.eapi_positive_config_test(func, cmds)

    def test_set_mac_address_with_disable(self):
        func = function('set_mac_address', disable=True)
        cmds = 'no ip virtual-router mac-address 00:11:22:33:44:55'
        self.eapi_positive_config_test(func, cmds)

    def test_set_mac_address_with_no_value(self):
        with self.assertRaises(ValueError):
            self.instance.set_mac_address(mac_address=None)

    def test_set_mac_address_with_bad_value(self):
        with self.assertRaises(ValueError):
            self.instance.set_mac_address(mac_address='0011.2233.4455')

    def test_set_mac_address_with_default(self):
        func = function('set_mac_address', default=True)
        cmds = 'default ip virtual-router mac-address 00:11:22:33:44:55'
        self.eapi_positive_config_test(func, cmds)


class TestApiVarpInterfaces(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiVarpInterfaces, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.varp.VarpInterfaces(None)
        self.config = open(get_fixture('running_config.varp')).read()

    def test_get_with_no_interface(self):
        self.config = ""
        self.setUp()
        result = self.instance.get('Vlan1000')
        self.assertIsNone(result)

    def test_add_address_with_value(self):
        func = function('set_addresses', 'Vlan4001', addresses=['1.1.1.4'])
        cmds = ['interface Vlan4001', 'no ip virtual-router address 1.1.1.2',
                'ip virtual-router address 1.1.1.4']
        self.eapi_positive_config_test(func, cmds)

    def test_add_address_when_interface_does_not_exist(self):
        self.config = ""
        self.setUp()
        func = function('set_addresses', 'Vlan10', addresses=['1.1.1.4'])
        cmds = ['interface Vlan10', 'ip virtual-router address 1.1.1.4']
        self.eapi_positive_config_test(func, cmds)

    def test_add_address_with_no_value(self):
        func = function('set_addresses', 'Vlan4002')
        cmds = ['interface Vlan4002', 'no ip virtual-router address']
        self.eapi_positive_config_test(func, cmds)

    def test_add_address_with_empty_list(self):
        func = function('set_addresses', 'Vlan4001', addresses=[])
        cmds = ['interface Vlan4001', 'no ip virtual-router address 1.1.1.2']
        self.eapi_positive_config_test(func, cmds)

    def test_add_address_with_default(self):
        func = function('set_addresses', 'Vlan4001', default=True)
        cmds = ['interface Vlan4001', 'default ip virtual-router address']
        self.eapi_positive_config_test(func, cmds)

    def test_add_address_with_disable(self):
        func = function('set_addresses', 'Vlan4001', disable=True)
        cmds = ['interface Vlan4001', 'no ip virtual-router address']
        self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, function
from testlib import EapiConfigUnitTest

import pyeapi.api.ntp


class TestApiNtp(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiNtp, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.ntp.Ntp(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_instance(self):
        result = pyeapi.api.ntp.instance(None)
        self.assertIsInstance(result, pyeapi.api.ntp.Ntp)

    def test_get(self):
        result = self.instance.get()
        ntp = {'servers': [{'1.2.3.4': 'prefer'},
                           {'10.20.30.40': None},
                           {'11.22.33.44': None},
                           {'123.33.22.11': 'prefer'},
                           {'123.44.55.66': None},
                           {'joe': None}],
               'source_interface': 'Loopback1'}
        keys = ['source_interface', 'servers']
        self.assertEqual(sorted(keys), sorted(result.keys()))
        self.assertEqual(ntp['source_interface'], result['source_interface'])
        self.assertIsNotNone(result['servers'])

    def test_create(self):
        cmd = 'ntp source Ethernet2'
        func = function('create', 'Ethernet2')
        self.eapi_positive_config_test(func, cmd)

    def test_delete(self):
        cmd = 'no ntp source'
        func = function('delete')
        self.eapi_positive_config_test(func, cmd)

    def test_default(self):
        cmd = 'default ntp source'
        func = function('default')
        self.eapi_positive_config_test(func, cmd)

    def test_set_source_interface(self):
        cmd = 'ntp source Vlan50'
        func = function('set_source_interface', 'Vlan50')
        self.eapi_positive_config_test(func, cmd)

    def test_add_server(self):
        cmd = 'ntp server 1.1.1.1'
        func = function('add_server', '1.1.1.1')
        self.eapi_positive_config_test(func, cmd)

    def test_add_server_prefer(self):
        cmd = 'ntp server 1.1.1.1 prefer'
        func = function('add_server', '1.1.1.1', prefer=True)
        self.eapi_positive_config_test(func, cmd)

    def test_add_server_invalid(self):
        func = function('add_server', '', prefer=True)
        self.eapi_exception_config_test(func, ValueError)

        func = function('add_server', ' ', prefer=True)
        self.eapi_exception_config_test(func, ValueError)

    def test_remove_server(self):
        cmd = 'no ntp server 1.1.1.1'
        func = function('remove_server', '1.1.1.1')
        self.eapi_positive_config_test(func, cmd)

    def test_remove_all_servers(self):
        cmd = 'no ntp'
        func = function('remove_all_servers')
        self.eapi_positive_config_test(func, cmd)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from systestlib import DutSystemTest

VIRT_ENTRY_A = 'ip virtual-router mac-address 00:11:22:33:44:55'
VIRT_ENTRY_B = 'ip virtual-router mac-address 00:11:22:33:44:56'
VIRT_ENTRY_C = 'ip virtual-router mac-address 00:11:22:33:44:57'
IP_CMD = 'ip virtual-router address'


class TestApiVarp(DutSystemTest):

    def _null_virtual_mac_command(self, dut):
        # Default format of ip virtual-router mac-address changed in EOS 4.17.x
        # pre 4.17.x    - no ip virtual-router mac-address
        # 4.17.x and on - ip virtual-router mac-address 00:00:00:00:00:00
        mac_dict = dut.api('varp')._parse_mac_address()
        virt_null_base = 'no ip virtual-router mac-address'
        if mac_dict['mac_address']:
            return virt_null_base + ' ' + mac_dict['mac_address']
        return virt_null_base

    def test_basic_get(self):
        for dut in self.duts:
            virt_null = self._null_virtual_mac_command(dut)
            dut.config([virt_null])
            response = dut.api('varp').get()
            self.assertIsNotNone(response)

    def test_get_with_value(self):
        for dut in self.duts:
            virt_null = self._null_virtual_mac_command(dut)
            dut.config([virt_null, VIRT_ENTRY_A])
            response = dut.api('varp').get()
            self.assertIsNotNone(response)
            self.assertEqual(response['mac_address'], '00:11:22:33:44:55')

    def test_get_none(self):
        # None response is for code versions before 4.17.x due to default
        # configuration change in 4.17.x. Default configuration difference
        # shown above in _null_virtual_mac_command
        for dut in self.duts:
            virt_null = self._null_virtual_mac_command(dut)
            dut.config([virt_null])
            response = dut.api('varp').get()
            self.assertIsNotNone(response)
            self.assertIn(response['mac_address'], [None, '00:00:00:00:00:00'])

    def test_set_mac_address_with_value(self):
        for dut in self.duts:
            virt_null = self._null_virtual_mac_command(dut)
            dut.config([virt_null])
            api = dut.api('varp')
            self.assertNotIn(VIRT_ENTRY_A, api.config)
            result = dut.api('varp').set_mac_address('00:11:22:33:44:55')
            self.assertTrue(result)
            self.assertIn(VIRT_ENTRY_A, api.config)

    def test_change_mac_address(self):
        for dut in self.duts:
            virt_null = self._null_virtual_mac_command(dut)
            dut.config([virt_null, VIRT_ENTRY_A])
            api = dut.api('varp')
            self.assertIn(VIRT_ENTRY_A, api.config)
            result = dut.api('varp').set_mac_address('00:11:22:33:44:56')
            self.assertTrue(result)
            self.assertIn(VIRT_ENTRY_B, api.config)

    def test_remove_mac_address(self):
        for dut in self.duts:
            virt_null = self._null_virtual_mac_command(dut)
            dut.config([virt_null, VIRT_ENTRY_A])
            api = dut.api('varp')
            self.assertIn(VIRT_ENTRY_A, api.config)
            result = dut.api('varp').set_mac_address(disable=True)
            self.assertTrue(result)
            self.assertNotIn(VIRT_ENTRY_A, api.config)

    def test_set_mac_address_with_bad_value(self):
        for dut in self.duts:
            virt_null = self._null_virtual_mac_command(dut)
            dut.config([virt_null])
            api = dut.api('varp')
            self.assertNotIn(VIRT_ENTRY_A, api.config)

            with self.assertRaises(ValueError):
                dut.api('varp').set_mac_address('0011.2233.4455')


class TestApiVarpInterfaces(DutSystemTest):

    def test_set_virtual_addr_with_values_clean(self):
        for dut in self.duts:
            dut.config(['no interface Vlan1000', 'interface Vlan1000',
                        'ip address 1.1.1.1/24'])
            api = dut.api('varp')
            self.assertNotIn('ip virtual-router address 1.1.1.2',
                             api.get_block('interface Vlan1000'))
            result = dut.api('varp').interfaces.set_addresses('Vlan1000',
                                                              ['1.1.1.2',
                                                               '1.1.1.3'])
            self.assertTrue(result)
            self.assertIn('ip virtual-router address 1.1.1.2',
                          api.get_block('interface Vlan1000'))
            self.assertIn('ip virtual-router address 1.1.1.3',
                          api.get_block('interface Vlan1000'))

    def test_set_virtual_addr_with_values_dirty(self):
        for dut in self.duts:
            dut.config(['no interface Vlan1000', 'interface Vlan1000',
                        'ip address 1.1.1.1/24',
                        'ip virtual-router address 1.1.1.20'])
            api = dut.api('varp')
            self.assertIn('ip virtual-router address 1.1.1.20',
                          api.get_block('interface Vlan1000'))
            result = dut.api('varp').interfaces.set_addresses('Vlan1000',
                                                              ['1.1.1.2',
                                                               '1.1.1.3'])
            self.assertTrue(result)
            self.assertIn('ip virtual-router address 1.1.1.2',
                          api.get_block('interface Vlan1000'))
            self.assertIn('ip virtual-router address 1.1.1.3',
                          api.get_block('interface Vlan1000'))
            self.assertNotIn('ip virtual-router address 1.1.1.20',
                             api.get_block('interface Vlan1000'))

    def test_default_virtual_addrs(self):
        for dut in self.duts:
            dut.config(['no interface Vlan1000', 'interface Vlan1000',
                        'ip address 1.1.1.1/24',
                        'ip virtual-router address 1.1.1.20',
                        'ip virtual-router address 1.1.1.21'])
            api = dut.api('varp')
            self.assertIn('ip virtual-router address 1.1.1.20',
                          api.get_block('interface Vlan1000'))
            self.assertIn('ip virtual-router address 1.1.1.21',
                          api.get_block('interface Vlan1000'))
            result = dut.api('varp').interfaces.set_addresses('Vlan1000',
                                                              default=True)
            self.assertTrue(result)
            self.assertNotIn('ip virtual-router address 1.1.1.20',
                             api.get_block('interface Vlan1000'))
            self.assertNotIn('ip virtual-router address 1.1.1.21',
                             api.get_block('interface Vlan1000'))

    def test_negate_virtual_addrs(self):
        for dut in self.duts:
            dut.config(['no interface Vlan1000', 'interface Vlan1000',
                        'ip address 1.1.1.1/24',
                        'ip virtual-router address 1.1.1.20',
                        'ip virtual-router address 1.1.1.21'])
            api = dut.api('varp')
            self.assertIn('ip virtual-router address 1.1.1.20',
                          api.get_block('interface Vlan1000'))
            self.assertIn('ip virtual-router address 1.1.1.21',
                          api.get_block('interface Vlan1000'))
            result = dut.api('varp').interfaces.set_addresses('Vlan1000',
                                                              addresses=None)
            self.assertTrue(result)
            self.assertNotIn('ip virtual-router address 1.1.1.20',
                             api.get_block('interface Vlan1000'))
            self.assertNotIn('ip virtual-router address 1.1.1.21',
                             api.get_block('interface Vlan1000'))

    def test_negate_virtual_addrs_with_disable(self):
        for dut in self.duts:
            dut.config(['no interface Vlan1000', 'interface Vlan1000',
                        'ip address 1.1.1.1/24',
                        'ip virtual-router address 1.1.1.20',
                        'ip virtual-router address 1.1.1.21'])
            api = dut.api('varp')
            self.assertIn('ip virtual-router address 1.1.1.20',
                          api.get_block('interface Vlan1000'))
            self.assertIn('ip virtual-router address 1.1.1.21',
                          api.get_block('interface Vlan1000'))
            result = dut.api('varp').interfaces.set_addresses('Vlan1000',
                                                              disable=True)
            self.assertTrue(result)
            self.assertNotIn('ip virtual-router address 1.1.1.20',
                             api.get_block('interface Vlan1000'))
            self.assertNotIn('ip virtual-router address 1.1.1.21',
                             api.get_block('interface Vlan1000'))

    def test_empty_list_virtual_addrs(self):
        for dut in self.duts:
            dut.config(['no interface Vlan1000', 'interface Vlan1000',
                        'ip address 1.1.1.1/24',
                        'ip virtual-router address 1.1.1.20',
                        'ip virtual-router address 1.1.1.21'])
            api = dut.api('varp')
            self.assertIn('ip virtual-router address 1.1.1.20',
                          api.get_block('interface Vlan1000'))
            self.assertIn('ip virtual-router address 1.1.1.21',
                          api.get_block('interface Vlan1000'))
            result = dut.api('varp').interfaces.set_addresses('Vlan1000',
                                                              addresses=[])
            self.assertTrue(result)
            self.assertNotIn('ip virtual-router address 1.1.1.20',
                             api.get_block('interface Vlan1000'))
            self.assertNotIn('ip virtual-router address 1.1.1.21',
                             api.get_block('interface Vlan1000'))

    def test_no_attr_virtual_addrs(self):
        for dut in self.duts:
            dut.config(['no interface Vlan1000', 'interface Vlan1000',
                        'ip address 1.1.1.1/24',
                        'ip virtual-router address 1.1.1.20',
                        'ip virtual-router address 1.1.1.21'])
            api = dut.api('varp')
            self.assertIn('ip virtual-router address 1.1.1.20',
                          api.get_block('interface Vlan1000'))
            self.assertIn('ip virtual-router address 1.1.1.21',
                          api.get_block('interface Vlan1000'))
            result = dut.api('varp').interfaces.set_addresses('Vlan1000')
            self.assertTrue(result)
            self.assertNotIn('ip virtual-router address 1.1.1.20',
                             api.get_block('interface Vlan1000'))
            self.assertNotIn('ip virtual-router address 1.1.1.21',
                             api.get_block('interface Vlan1000'))


if __name__ == '__main__':
    unittest.main()
